
#include "display_levels.h"


int main(){

    Sandbox sandbox;
    char *sandbox_conf_file = (char*)"./sandbox_conf.yaml";

    if(sandbox.loadConfig(sandbox_conf_file)){
        std::cout << "Failed to load the configuration" << std::endl;
        return 1;
    }

    
    if(sandbox.init()){
        std::cout << "Failed to initilize sandbox" << std::endl;
        return 1;
    }

    displayLevels( sandbox, sandbox.getProjection()->getDistanceTopSandbox(), 0.14f );
}


/*
*   Show levels of the sandbox in color
*   top => distance from camera to the top of the sandbox
*/
void displayLevels(Sandbox &sandbox, float top, float height){
        
    // Needed because the values retrive from depth frames can change a bit although nothing has changed physically
    const float DEPTH_MARGIN_ERROR = 0.014f;

    char windowName[] = "Sandbox";
    cv::namedWindow(windowName, CV_WINDOW_NORMAL);
    cv::setWindowProperty(windowName, CV_WND_PROP_FULLSCREEN, CV_WINDOW_FULLSCREEN);
    
    sandbox.captureFrame();
    cv::Mat1f depth_cache = sandbox.getDepthFrame(); 
    cv::Mat1f new_depth = cv::Mat(depth_cache.size(), CV_32F);
    cv::Mat3b colorized = cv::Mat(depth_cache.size(), CV_8UC3);
    cv::Mat3b res = cv::Mat(depth_cache.size(), CV_8UC3);

    do{
        sandbox.captureFrame();
        new_depth = sandbox.getDepthFrame();
        // don't show what is above the top of sandox
        depth_cache.copyTo( new_depth, (new_depth < top - 0.1f) );
        // don't update little differences due to the camera depth captors
        new_depth.copyTo( depth_cache, (cv::abs(depth_cache-new_depth) > DEPTH_MARGIN_ERROR/2) );

        // TODO : try to colorize with HSV
        colorized = colorizeDepth(depth_cache, top, height);
        res = sandbox.adjustProjection(colorized, depth_cache);

        cv::cvtColor(res, res, CV_RGB2BGR);
        cv::imshow(windowName, res);

    } while (cv::waitKey(10) != ESCAPE_CHAR);

    cv::destroyAllWindows();

}


/*
*   Colorize depth frame
*/
cv::Mat colorizeDepth(cv::Mat depth, float sandboxTop, float sandboxHeight){

    // Matrix with depth from 0 to n, where 0 is the lowest point
    cv::Mat normalizedDepth = depth * -1.0f + (sandboxTop + sandboxHeight);
    normalizedDepth.setTo(0, normalizedDepth < 0);
    normalizedDepth.setTo(sandboxHeight, normalizedDepth > sandboxHeight);
    cv::Mat res = cv::Mat(depth.rows, depth.cols, CV_8UC3);
    
    
    for(int j=0; j<depth.rows; j++){
        for(int i=0; i<depth.cols; i++){

            res.at<cv::Vec3b>(j,i) = floatToColor(normalizedDepth.at<float>(j,i), 0.0f, sandboxHeight);
        }
    }

    return res;
}



/*
*   Return the color matching the value in the gradient color between min and max
*   Where :
*       red = highest point
*       blue = lowest point
*   
*   Color order :
*       blue > cyan > green > yellow > red
*/
cv::Vec3b floatToColor(float value, float min, float max){

    const uint CHANNEL_MAX = 255;

    // Map for the color gradient (rgb channels)
    uint initColors[4][3] = { {           0,           0, CHANNEL_MAX },
                              {           0, CHANNEL_MAX, CHANNEL_MAX },
                              {           0, CHANNEL_MAX,           0 },
                              { CHANNEL_MAX, CHANNEL_MAX,           0 } };
    
    // Indicates how the value changes :
    //  - when reducing channel value   : 255 + (-1 * (value % 255))
    //  - when increasing channel value :   0 + ( 1 * (value % 255))
    int coeffColors[4][3] = { {  0,  1,  0 },
                              {  0,  0, -1 },
                              {  1,  0,  0 },
                              {  0, -1,  0 } };
    
    // values go from min to max
    float clamped = value;
    clamped = (clamped < min) ? min : clamped;
    clamped = (clamped > max) ? max : clamped;

    // values go from 0 to relativeMax
    float relative = clamped - min;
    float relativeMax = max - min;

    const uint COLOR_MAX = 4*(CHANNEL_MAX+1)-1;
    uint color = static_cast<uint>(relative * COLOR_MAX / relativeMax);
    uint index = static_cast<uint>(color / (CHANNEL_MAX+1));
    
    // red at the highest point, blue at the lowest
    uint8_t r = static_cast<uint8_t>( initColors[index][0] + coeffColors[index][0] * (color % (CHANNEL_MAX+1)) );
    uint8_t g = static_cast<uint8_t>( initColors[index][1] + coeffColors[index][1] * (color % (CHANNEL_MAX+1)) );
    uint8_t b = static_cast<uint8_t>( initColors[index][2] + coeffColors[index][2] * (color % (CHANNEL_MAX+1)) );

    return cv::Vec3b(r,g,b);
}
